<?php


namespace Repository;


use Doctrine\ORM\EntityManager;
use Exception;
use Util\InstanceProvider;
use Teknisa\Model\Traits\TQuery;
use Zeedhi\DevOps\Util\Utilities;

class WorkItem
{
    use TQuery;
    /**
     * @var EntityManager
     */
    private $entityManager;

    /**
     * WorkItem constructor.
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function findLevelPattern()
    {
        $query = <<<SQL
            SELECT
                   T.* 
              FROM PARAMGERAL P, 
                   TABLE(MASCARA_NIVEL(P.CDPICTITTRAB)) T
SQL;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query)['data'];
    }

    public function find($params)
    {
        if (!isset($params['CDPARENT_ITTRAB'])) {
            $search = $_REQUEST['search'];
            $searchIn = array(
                'IT.CDARVITTRAB',
                'IT.NMITEMTRAB',
                'IT.CDREDUITTRAB',
                'IT.NMITTRABESPA',
                'IT.NMITTRABINGL',
            );
            $filters = 'AND (';
            end($searchIn);         // move the internal pointer to the end of the array
            $lastKey = key($searchIn);

            foreach ($searchIn as $index => $column) {
                $filters .= "(UPPER($column) LIKE UPPER('%$search%'))";
                if ($lastKey != $index) {
                    $filters .= " OR ";
                }
            }
            $filters .= ')';
        } else if ($params['CDPARENT_ITTRAB'] == "NULL") {
            $filters = "AND IP.CDITEMTRAB IS NULL AND IT.CDNVITEMTRAB = 1";
        } else {
            $filters = "AND IP.CDITEMTRAB = :PARENT_ID";
        }

        $query = <<<SQL
        SELECT
                IT.CDITEMTRAB  ,                                            -- ID (Novo código -> CDCONTADOR = "ITEMTRAB")
                IT.CDARVITTRAB ,                                            -- "Código"
                FORMATITTRABMASK(IT.CDARVITTRAB) AS CDARVITTRAB_MK,         -- "Código" (com máscara, se for preciso)
                IT.NMITEMTRAB  ,                                            -- "Nome"
                IT.CDNVITEMTRAB,                                            -- Nível da árvore (necessário pra manter compatibilidade com Delphi)
                IT.CDREDUITTRAB,                                            -- "Código reduzido"
                IT.NMITTRABESPA,                                            -- "Espanhol"
                IT.NMITTRABINGL,                                            -- "Inglês"
                IT.CDTIPOITTRAB,                                            -- Cód. do Tipo de item de trabalho (relacionamento com tabela TIPOITTRAB
                TP.DSTIPOITTRAB,                                            -- "Tipo Item"
                IT.IDATIVO,                                                 -- Ativo (S/N)
                IP.CDITEMTRAB AS CDPARENT_ITTRAB,                           -- PARENT ID
                IP.NMITEMTRAB AS NMPARENT_ITTRAB,                           -- Nome do registro pai   
                IP.CDARVITTRAB AS CDPARENT_ARVITTRAB,                       -- Código do registro pai
                FORMATITTRABMASK(IP.CDARVITTRAB) AS CDPARENT_ARVITTRAB_MK   -- Código do registro pai (com máscara, se for preciso)
            FROM
                ITEMTRAB IT, TIPOITTRAB TP, ITEMTRAB IP, PARAMGERAL PG, TABLE(MASCARA_NIVEL(PG.CDPICTITTRAB)) MK   
            WHERE (TP.CDTIPOITTRAB(+) = IT.CDTIPOITTRAB)
            AND (TO_NUMBER(MK.NIVEL(+)) = TO_NUMBER(IT.CDNVITEMTRAB)-1)
            AND (IP.CDARVITTRAB(+) = SUBSTR(IT.CDARVITTRAB, 1, MK.TAMANHO_ACUMULADO_NIVEL))
                $filters
        ORDER BY IT.CDARVITTRAB
SQL;

        $params = array(
            "PARENT_ID" => isset($params['CDPARENT_ITTRAB']) ? $params['CDPARENT_ITTRAB'] : null
        );

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }

    public function findMasterItemSelectItems()
    {
        $query = <<<SQL
            SELECT
                   IT.CDITEMTRAB  ,                                            -- ID (Novo código -> CDCONTADOR = "ITEMTRAB")
                   IT.CDARVITTRAB ,                                            -- "Código"
                   FORMATITTRABMASK(IT.CDARVITTRAB) AS CDARVITTRAB_MK,         -- "Código" (com máscara, se for preciso)
                   IT.NMITEMTRAB,                                              -- "Nome"
                   IT.CDNVITEMTRAB,
                   IP.CDITEMTRAB AS CDPARENT_ITTRAB
              FROM
                   ITEMTRAB IT, TIPOITTRAB TP, ITEMTRAB IP, PARAMGERAL PG, TABLE(MASCARA_NIVEL(PG.CDPICTITTRAB)) MK   
             WHERE (TP.CDTIPOITTRAB(+) = IT.CDTIPOITTRAB)
               AND (TO_NUMBER(MK.NIVEL(+)) = TO_NUMBER(IT.CDNVITEMTRAB)-1)
               AND (IP.CDARVITTRAB(+) = SUBSTR(IT.CDARVITTRAB, 1, MK.TAMANHO_ACUMULADO_NIVEL))
               AND TO_NUMBER(NVL(IT.CDNVITEMTRAB, 0)) < 4
               AND ((IP.CDITEMTRAB IS NULL AND TO_NUMBER(NVL(IT.CDNVITEMTRAB, 0)) = 1) 
                 OR (IP.CDITEMTRAB IS NOT NULL AND TO_NUMBER(NVL(IT.CDNVITEMTRAB, 0)) > 1))
          ORDER BY IT.CDARVITTRAB
SQL;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query);
    }

    public function findWorkItemType()
    {
        $query = <<<SQL
            SELECT CDTIPOITTRAB, DSTIPOITTRAB 
              FROM TIPOITTRAB 
             WHERE IDATIVO = 'S'
SQL;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query);
    }

    public function findWorkItemByCodeAndLevel($cdArvItTrab, $cdNvItemTrab)
    {
        $query = <<<SQL
            SELECT CDITEMTRAB 
              FROM ITEMTRAB 
             WHERE CDARVITTRAB  = :CDARVITTRAB
               AND CDNVITEMTRAB = :CDNVITEMTRAB
SQL;
        $params = array (
            'CDARVITTRAB'  => $cdArvItTrab,
            'CDNVITEMTRAB' => $cdNvItemTrab,
        );

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }

    public function insert($cdItemTrab, $nmItemTrab, $cdArvItTrab, $cdNvItemTrab, $nmItTrabEspa, $nmItTrabIngl, $cdReduItTrab, $cdTipoItTrab, $nrOrg, $cdOperador)
    {
        $query = <<<SQL
            INSERT INTO ITEMTRAB (CDITEMTRAB, NMITEMTRAB, CDARVITTRAB, CDNVITEMTRAB, NMITTRABESPA, NMITTRABINGL, CDREDUITTRAB, CDTIPOITTRAB, 
                                  NRORG, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, IDATIVO)
                          VALUES (:CDITEMTRAB, :NMITEMTRAB, :CDARVITTRAB, :CDNVITEMTRAB, :NMITTRABESPA, :NMITTRABINGL, :CDREDUITTRAB, :CDTIPOITTRAB,
                                  :NRORG, SYSDATE, SYSDATE, :NRORG, :CDOPERADOR, :NRORG, :CDOPERADOR, 'S')
SQL;
        $params = array(
            'CDITEMTRAB'   => $cdItemTrab,
            'NMITEMTRAB'   => $nmItemTrab,
            'CDARVITTRAB'  => $cdArvItTrab,
            'CDNVITEMTRAB' => $cdNvItemTrab,
            'NMITTRABESPA' => $nmItTrabEspa,
            'NMITTRABINGL' => $nmItTrabIngl,
            'CDREDUITTRAB' => $cdReduItTrab,
            'CDTIPOITTRAB' => $cdTipoItTrab,
            'NRORG'        => $nrOrg,
            'CDOPERADOR'   => $cdOperador
        );

        return $this->entityManager->getConnection()->executeQuery($query, $params);
    }

    public function update($cdItemTrab, $nmItemTrab, $nmItTrabEspa, $nmItTrabIngl, $cdReduItTrab, $cdTipoItTrab, $idAtivo, $nrOrg, $cdOperador)
    {
        $query = <<<SQL
            UPDATE ITEMTRAB
               SET NMITEMTRAB   = :NMITEMTRAB,
                   CDREDUITTRAB = :CDREDUITTRAB,
                   NMITTRABESPA = :NMITTRABESPA,
                   NMITTRABINGL = :NMITTRABINGL,
                   CDTIPOITTRAB = :CDTIPOITTRAB,
                   IDATIVO      = :IDATIVO,
                   DTULTATU     = SYSDATE,
                   CDOPERULTATU = :CDOPERULTATU
             WHERE CDITEMTRAB = :CDITEMTRAB
               AND NRORG      = :NRORG
                   
SQL;
        $params = array(
            'NMITEMTRAB'   => $nmItemTrab,
            'CDREDUITTRAB' => $cdReduItTrab,
            'NMITTRABESPA' => $nmItTrabEspa,
            'NMITTRABINGL' => $nmItTrabIngl,
            'CDTIPOITTRAB' => $cdTipoItTrab,
            'IDATIVO'      => $idAtivo,
            'CDOPERULTATU' => $cdOperador,
            'CDITEMTRAB'   => $cdItemTrab,
            'NRORG'        => $nrOrg
        );

        return $this->entityManager->getConnection()->executeQuery($query, $params);
    }

    public function findItemtrab(){
        $query = <<<SQL
            SELECT CDREDUITTRAB, NMITEMTRAB, CDITEMTRAB, NRCPLXITTRAB
              FROM ITEMTRAB
             WHERE NVL(IDATIVO,'S') = 'S'
             AND CDNVITEMTRAB = '4'
          ORDER BY NMITEMTRAB
SQL;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query);
    }
}
